/*
 * Search for files containing a given string.
 * $Id: scanfiles.cpp,v 1.3 2002/08/29 16:02:08 bruce Exp $
 *
 * $Log: scanfiles.cpp,v $
 * Revision 1.3  2002/08/29 16:02:08  bruce
 *
 * Updated TODO list
 *
 * Revision 1.2  2002/08/29 15:56:42  bruce
 *
 * Various changes/bug fixes for release
 *
 *
 * ToDo:
 * 1. Smarter searching of HTML files
 * 2. Smarter searching of Hancomword files
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>
#include "zearch.h"

char *dirlist[] = { 
	"~/Documents",
	"/usr/mnt.rom/cf",
	"/usr/mnt.rom/card",
	NULL
};

char *pimlist[] = {
	"~/Applications/addressbook/addressbook.xml",
	"~/Applications/datebook/datebook.xml",
	"~/Applications/todolist/todolist.xml",
	NULL
};

// Different searchable document types defined here
#define M_HTML 1
#define M_TEXT 2
#define M_EXCEL 3
#define M_HANCOMSHEET 4
#define M_HANCOMWORD 5
#define M_MSWORD 6
#define M_RTF 7



char *homedir;
char *defhome = "/home/root";

Zearch *zptr;

/*
 * fix special escaped characters that might appear in the input
 */
void
fixchars(char *s)
{
	char *cp, *xp;

	/*
	 * &amp;
	 * &quot;
	 * &gt;
	 * &lt;
	 */
	while( (cp = strstr(s, "&amp;")) ) {
		*cp++ = '&';
		xp = cp + 4;
		strcpy(cp,xp);
	}
	while( (cp = strstr(s, "&quot;")) ) {
		*cp++ = '"';
		xp = cp + 5;
		strcpy(cp,xp);
	}
	while( (cp = strstr(s, "&gt;")) ) {
		*cp++ = '>';
		xp = cp + 3;
		strcpy(cp,xp);
	}
	while( (cp = strstr(s, "&lt;")) ) {
		*cp++ = '<';
		xp = cp + 3;
		strcpy(cp,xp);
	}
}

int 
getattr( FILE *fp, char *name, char *value )
{
	int c;
	char *cp;

	/* skip to start of attribute name */
	while( (c = fgetc(fp)) != EOF ) {
		if( isspace(c) ) continue;
		if( '/' == c ) { /* possible end of tag */
			c = fgetc(fp);
			if( c == EOF ) return 0;
			if( '>' == c ) return 0;  /* end of tag, no more attributes */
			ungetc(c,fp);
			break;
		}
		else break;
	}
	/* get name */
	cp = name;
	*cp++ = c;
	while( (c = fgetc(fp)) != EOF ) {
		if( c == '=' ) { /* end of name */
			*cp = '\0';
			break;
		}
		if( isspace(c) ) { /* also end of name */
			*cp = '\0';
			/* skip to '=' */
			while( (c = fgetc(fp)) != EOF && c != '=' ) ;
		}
		*cp++ = c;
	}
	*cp = '\0';

	/* get value */
	cp = value;
	*cp = '\0';
	while( (c = fgetc(fp)) != EOF && c != '"' ); /* skip to quote */
	
	while( (c = fgetc(fp)) != EOF ) {
		if( c == '"' ) { /* end of value */
			*cp = '\0';
			break;
		}
		*cp++ = c;
	}
	*cp = '\0';
	fixchars(value);
	if(strlen(name)) return 1;
	else return 0;
}


int 
findtag( FILE *fp, char *tagname )
{
	int c;
	char *cp;

	cp = tagname;
	while( (c = fgetc(fp)) != EOF ) {
		if('<' == c) { /* start of a tag */
			cp = tagname;
			while( (c = fgetc(fp)) != EOF ) {
				if( *cp && c == *cp ) {
					cp++;
					if( ! *cp ) { /* end of tagname, found a match */
						c = fgetc(fp);
						if( isspace(c) ) return 1;
						else break; /* this tagname is a superset of what we want */
					}
				}
			}
		}
	}
	return 0;

}

/* Poor-mans string searcher for MSWORD files
 */
int mswordcmp(char *str1, const char *str2, int len1, int len2)
{
	while(len1 && len2) {
		if( ! *str1 ) {
			str1++; /* skip null byte of unicode char */
			len1--;
		}
		if(tolower(*str1)  != tolower(*str2) ) {
				return (*str1 - *str2);  /* no match */
		}
		--len1;
		--len2;
		str1++;
		str2++;
	}
	if( 0 == len2 ) return 0;
	else return 1;
}


/* 
 * scan a file for a string 'str'
 * dfname is the name of the .destop file
 * fname is the name of the file to search
 * sname is the name of the document as displayed in the Documents tab
 */
int scanfile(char *dfname, char *fname, int mtype, char *sname, const char *str)
{
	FILE *fp;
	char *cp1, *cp2;
	int slen, n;
	int found;
	int mflag;

	char buf[4096];

	found = 0;
	fp = fopen(fname,"r");
	if( ! fp ) {
		fprintf(stderr, "scanfile: Can't open file: %s\n",fname);
		return 0;
	}

	slen = strlen(str);
	while( !found && (n = fread(buf,1,sizeof(buf)-1,fp)) ) {
		for(cp1 = &buf[0]; cp1 <= &buf[n-slen]; cp1++) {
			if(mtype == M_MSWORD) {
				mflag = mswordcmp(cp1,str,(int)(&buf[n] - cp1),slen);
			}
			else {
				mflag = strncasecmp(cp1,str,slen);
			}
			if( mflag == 0 ) { /* found a match!! */
				int i;
				/* printf("%s:\n",fname); */
				cp2 = cp1;

				/* backup to show some context of the match */
				i = 0;
				while(*cp2 && isprint(*cp2) && cp2 > &buf[0]) {
					i++;
					/* try to backup to a word boundry at least 10 
					 * characters before the match, but don't go
					 * back more than 20 characters
					 */
					if( i > 40 || (i > 30 && isspace(*cp2)) ) break;
					cp2--;
				}

				/* go forward to show some context of the match */
				cp1 += slen; /* point to the end of the matching word */
				i = 0; 
				while(*cp1 && isprint(*cp1) && cp1 < &buf[n-1]) {
					i++;
					/* try to go forward to a word boundry at least
					 * 20 characters before the match, but don't go
					 * more than 30 characters past the matching word.
					 */
					if( i> 40 || (i > 30 && isspace(*cp1)) ) break;
					cp1++;
				}

				*cp1 = '\0'; /* mark end of string */
				while( *cp2 && isspace(*cp2) ) cp2++; /* trim leading spaces */
				slen = strlen(cp2);
				// if(slen > 40) cp2[40] = '\0'; /* set max length of 40 */
				cp1 = cp2;
				while( *cp1 ) { /* transform all whitespace to ' ' */
					if( isspace(*cp1) ) *cp1 = ' ';
					cp1++;
				}
				{ /* display 'name' (displayname) if specified, otherwise
					 display just the filename of the .desktop file */
					char zbuf[1024];
					char shortname[NAME_MAX + 1];
					char *np1,*np2;

					if(sname[0]) {
						np1 = sname;
					}
					else {
						strcpy(shortname,dfname);
						/* remove direcotory names */
						np1 = strrchr(shortname,'/');
						if(np1) {
							++np1;
						}
						else {
							np1 = shortname;
						}
						/* remove .desktop */
						np2 = strstr(np1,".desktop");
						if(np2) {
							*np2 = '\0';
						}
					}
					//sprintf(zbuf,"%s: %-30.30s\n",np1,cp2);
					sprintf(zbuf,"%s: %-.80s\n",np1,cp2);
					zptr->insert(zbuf,'d',dfname);

				}
#ifdef DEBUG
				printf("%-30.30s\n",cp2);  /* show match in some context */
#endif
				found = 1;
				break;
			}
		}
	}
	fclose(fp);
	if(found) return 1;
	else return 0;
}

int getvalue(char *line, char *value)
{
	char *cp;

	cp = strchr(line,'=');
	value[0] = '\0';
	if(cp) {
		int len;
		cp++; /* skip past '=' */
		while( isspace(*cp) ) cp++; /* skip spaces */
		strncpy(value,cp,NAME_MAX);
		len = strlen(value);
		len--;
		while( len && isspace(value[len]) ) {
			/* trim trailing whitespace */
			value[len--] = '\0';
		}
		return 1;
	}
	else {
		return 0;
	}
}


/*
 * given a named '.desktop' file, open it, get the real filename
 * and search the real file for 'str'
 */
int searchfile( char *filename, const char *str )
{
	FILE *fpd;
	char iline[4096];
	char rfilename[NAME_MAX+1];
	char ftype[NAME_MAX+1];
	char displayname[NAME_MAX+1];
	int mtype;  // mime type code of file to search (M_XXXX)

	fpd = fopen(filename,"r");
	if( ! fpd ) {
		fprintf(stderr, "searchfile: Can't open file: %s\n",filename);
		return 0;
	}
	if( NULL == fgets(iline,4095,fpd) ) {
		fprintf(stderr, "searchfile: Can't read file: %s\n",filename);
		return 0;
	}
	/* look for line with File = */
	while( fgets(iline,4095,fpd) ) {
		if( 0 == strncasecmp(iline,"File",4) ) {
			getvalue(iline,rfilename);
		}
		if( 0 == strncasecmp(iline,"Type",4) ) {
			getvalue(iline,ftype);
		}
		if( 0 == strncasecmp(iline,"Name",4) ) {
			getvalue(iline,displayname);
		}
	}
	fclose(fpd);
	/*
	printf("rfilename: %s\n",rfilename);
	printf("ftype: %s\n",ftype);
	*/
	/* almost ready to search file */
	mtype = 0;
	if( rfilename[0] && ftype[0] ) {
		if(  0 == strcasecmp(ftype,"text/html") ) {
			mtype = M_HTML;
		}
		else if(  0 == strncasecmp(ftype,"text/",5) ) {
			mtype = M_TEXT;
		}
		else if ( 0 == strcasecmp(ftype,"application/excel") ) {
			mtype = M_EXCEL;
		}
		else if ( 0 == strcasecmp(ftype,"application/hancomsheet") ) {
			mtype = M_HANCOMSHEET;
		}
		else if ( 0 == strcasecmp(ftype,"application/hancomword") ) {
			mtype = M_HANCOMWORD;
		}
		else if ( 0 == strcasecmp(ftype,"application/msword") ) {
			mtype = M_MSWORD;
		}
		else if ( 0 == strcasecmp(ftype,"application/rtf") ) {
			mtype = M_RTF;
		}
		/* add other valid mimetypes here */

		if(mtype > 0) {
			/* printf("OK to search file: %s\n",rfilename); */
			scanfile(filename,rfilename,mtype,displayname,str);
		}
	}
	return 1;
}


/*
 * Find all '.desktop files' and find the real filename and
 * mime type.  Search 'searchable' files for 'str'
 */

void
searchdir( char *dirname, const char *str )
{
	DIR *zdir;
	struct dirent * zdirent;
	char *fname;
	char dirfullname[NAME_MAX+1];

	// printf("Searching %s\n",dirname);

	zdir = opendir(dirname);
	if( ! zdir ) return;
	while( (zdirent = readdir( zdir )) 
			&& zptr->resultcount < ZEARCH_MAX_ITEMS ) {
		char *cp;
		struct stat statbuf;
		int rv;

		fname = zdirent->d_name;
		if( '.' == fname[0] ) continue; /* skip '.' files */
		/* build full path name */
		strcpy(dirfullname,dirname);
		strcat(dirfullname,"/");
		strcat(dirfullname,fname);
		/* see if it's a .desktop file */
		cp = rindex(fname,'.');
		if(cp && 0 == strcmp(cp,".desktop")) {
			/* found one */
			/* printf("Found file: %s\n",dirfullname); */
			searchfile(dirfullname,str);
		}
		else { /* see if directory */
			rv = stat(dirfullname,&statbuf);
			if( 0 == rv ) {
				if(statbuf.st_mode & S_IFDIR) {
					/* found subdirectory */
					searchdir(dirfullname,str);
				}
			}
		}
	}
	closedir(zdir);
}

/* scan XML address book */
int
scanxmladdr( char *fname, const char *str )
{
	FILE *fp;
	int found;
	char fileas[1024];
	char hphone[1024];
	char bphone[1024];
	char uid[1024];
	char rid[1024];
	char name[1024];
	char *cp1;
	int n,slen;

	char buf[4096];

	found = 0;
	slen = strlen(str);
	fp = fopen(fname,"r");
	if( ! fp ) {
		fprintf(stderr, "Can't open address book file: %s\n",fname);
		zptr->insert("No address book found!",'h',NULL);
		return 0;
	}

	while( findtag(fp, "Contact") ) {
		/* printf( "Found Contact tag\n"); */
		found = 0;
		fileas[0] = hphone[0] = bphone[0] = uid[0] = rid[0] = '\0';
		while( getattr(fp,name,buf) ) {
			/* printf("Found attr %s = %s\n",name,buf); */
			if( 0 == strcasecmp(name,"FileAs" ) ) {
				strcpy(fileas,buf);
			}
			if( 0 == strcasecmp(name,"HomePhone" ) ) {
				strcpy(hphone,buf);
			}
			if( 0 == strcasecmp(name,"BusinessPhone" ) ) {
				strcpy(bphone,buf);
			}
			if( 0 == strcasecmp(name,"Uid" ) ) {
				strcpy(uid,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"rid" ) ) {
				strcpy(rid,buf);
				continue;
			}
			/* we should now have a string to search in buf[] */
			n = strlen(buf);
			for(cp1 = &buf[0]; n >= slen && cp1 <= &buf[n-slen]; cp1++) {
				if( 0 == strncasecmp(cp1,str,slen) ) {
					/* found a match!! */
					/* printf("Match of %s found in %s\n",str,fname); */
					found = 1;
					break;  /* don't need to search this string anymore */
				}
			}
		}
		if(found) {
			if(bphone[0]) {
				cp1 = strchr(bphone,'\r');
				if( cp1 ) *cp1 = '\0';
				cp1 = strchr(bphone,'\n');
				if( cp1 ) *cp1 = '\0';
				sprintf(buf,"%s B",bphone);
			}
			else if( hphone[0] ) {
				cp1 = strchr(hphone,'\r');
				if( cp1 ) *cp1 = '\0';
				cp1 = strchr(hphone,'\n');
				if( cp1 ) *cp1 = '\0';
				sprintf(buf,"%s H",hphone);
			}
			else { /* no phone number to display */
				buf[0] = '\0';
			}
#ifdef DEBUG
			printf("%-20.20s %s\n",fileas, buf);
#endif
			{
				char zbuf[1024];
				//sprintf(zbuf,"%-30.30s %s\n",fileas, buf);
				sprintf(zbuf,"%-.80s %s\n",fileas, buf);
				zptr->insert(zbuf,'a',fname);
				if( zptr->resultcount >= ZEARCH_MAX_ITEMS ) {
					break;
				}
			}
		}
	}
	fclose(fp);
	if(found) return 1;
	else return 0;
}

/* scan XML datebook */
int
scanxmldatebk( char *fname, const char *str )
{
	FILE *fp;
	int found;
	char descrip[1024];
	char starttm[1024];
	char endtm[1024];
	char enddt[1024];
	char rtype[1024];
	char uid[1024];
	char rid[1024];
	char name[1024];
	char hasenddt;
	char rfreq;
	char *cp1;
	int n,slen;
	time_t xtime;

	char buf[4096];

	found = 0;
	slen = strlen(str);
	fp = fopen(fname,"r");
	if( ! fp ) {
		fprintf(stderr, "Can't open datebook file: %s\n",fname);
		zptr->insert("No calendar file found!",'h',NULL);
		return 0;
	}

	while( findtag(fp, "event") ) {
		/* printf( "Found Contact tag\n"); */
		found = 0;
		descrip[0] = starttm[0] = endtm[0] = uid[0] = rid[0] = '\0';
		rtype[0] = enddt[0] = hasenddt = rfreq = '\0';
		while( getattr(fp,name,buf) ) {
			/* printf("Found attr %s = %s\n",name,buf); */
			if( 0 == strcasecmp(name,"description" ) ) {
				strcpy(descrip,buf);
			}
			if( 0 == strcasecmp(name,"start" ) ) {
				strcpy(starttm,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"end" ) ) {
				strcpy(endtm,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"rtype" ) ) {
				strcpy(rtype,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"enddt" ) ) {
				strcpy(enddt,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"rhasenddate" ) ) {
				hasenddt = buf[0];
				continue;
			}
			if( 0 == strcasecmp(name,"rfreq" ) ) {
				rfreq = buf[0];
				continue;
			}
			if( 0 == strcasecmp(name,"Uid" ) ) {
				strcpy(uid,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"rid" ) ) {
				strcpy(rid,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"loud") ) continue;
			if( 0 == strcasecmp(name,"categories") ) continue;
			if( 0 == strcasecmp(name,"rinfo") ) continue;
			if( 0 == strcasecmp(name,"alarm") ) continue;

			/* we should now have a string to search in buf[] */
			n = strlen(buf);
			for(cp1 = &buf[0]; n >= slen && cp1 <= &buf[n-slen]; cp1++) {
				if( 0 == strncasecmp(cp1,str,slen) ) {
					/* found a match!! */
					/* printf("Match of %s found in %s\n",str,fname); */
					found = 1;
					break; /* don't need to search this string anymore */
				}
			}
		}
		if(found) {
			buf[0] = '\0';
			if(! descrip[0]) {
				strcpy(descrip,"Unnamed event");
			}
			if( rfreq == '1' ) { /* a repeating event */
				strcat(buf,rtype);
				strcat(buf," ");
			}
			if(starttm[0]) {
				/* TODO: if it's an all day event, don't display the time,
				 * but perhaps display 'All Day' before the date */
				xtime = (time_t) atol(starttm);
				strcat(buf,ctime(&xtime));
				n = strlen(buf);
				--n;
				/* trim trailing whitespace */
				while(n >= 0 && isspace(buf[n]) ) {
					buf[n--] = '\0';
				}

				if( hasenddt == '1' ) { /* and end date */
					strcat(buf," until ");
					xtime = (time_t) atol(enddt);
					{ /* remove 00:00:00 if it exists */
						char zzbuf[64], *tp;
						strcpy(zzbuf,ctime(&xtime));
						if( (tp = strstr(zzbuf,"00:00:00 ")) ) {
							char *tp1 = tp + 9;
							while(*tp) {
								*tp++ = *tp1++;
							}
						}
						strcat(buf,zzbuf);
					}
					n = strlen(buf);
					--n;
					/* trim trailing whitespace */
					while(n >= 0 && isspace(buf[n]) ) {
						buf[n--] = '\0';
					}
				}
			}
#ifdef DEBUG
			printf("%-20.20s %s\n",descrip, buf);
#endif
			{
				char zbuf[1024];
				//sprintf(zbuf,"%-30.30s %s\n",descrip, buf);
				sprintf(zbuf,"%-.80s %s\n",descrip, buf);
				zptr->insert(zbuf,'c',fname);
				if( zptr->resultcount >= ZEARCH_MAX_ITEMS ) {
					break;
				}
			}
		}
	}
	fclose(fp);
	if(found) return 1;
	else return 0;
}

/* scan XML todo */
int
scanxmltodo( char *fname, const char *str )
{
	FILE *fp;
	int found;
	char descrip[1024];
	char dateyr[1024];
	char datemon[1024];
	char dateday[1024];
	char completed;
	char priority;
	char hasdate;
	char uid[1024];
	char rid[1024];
	char name[1024];
	char *cp1;
	int n,slen;

	char buf[4096];

	found = 0;
	slen = strlen(str);
	fp = fopen(fname,"r");
	if( ! fp ) {
		fprintf(stderr, "Can't open file: %s\n",fname);
		zptr->insert("No ToDo list found!",'h',NULL);
		return 0;
	}

	while( findtag(fp, "Task") ) {
		/* printf( "Found Contact tag\n"); */
		found = 0;
		descrip[0] = dateyr[0] = datemon[0] = dateday[0] 
			= uid[0] = rid[0] = '\0';
		completed = priority = hasdate = '\0';
		while( getattr(fp,name,buf) ) {
			/* printf("Found attr %s = %s\n",name,buf); */
			if( 0 == strcasecmp(name,"Description" ) ) {
				strcpy(descrip,buf);
			}
			if( 0 == strcasecmp(name,"DateYear" ) ) {
				strcpy(dateyr,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"DateMonth" ) ) {
				strcpy(datemon,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"DateDay" ) ) {
				strcpy(dateday,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"Completed" ) ) {
				completed = buf[0];
				continue;
			}
			if( 0 == strcasecmp(name,"Priority" ) ) {
				priority = buf[0];
				continue;
			}
			if( 0 == strcasecmp(name,"HasDate" ) ) {
				hasdate = buf[0];
				continue;
			}
			if( 0 == strcasecmp(name,"Uid" ) ) {
				strcpy(uid,buf);
				continue;
			}
			if( 0 == strcasecmp(name,"rid" ) ) {
				strcpy(rid,buf);
				continue;
			}
			/* future: allow search of only uncompleted tasks */
			if( 0 == strcasecmp(name,"rinfo") ) continue;

			/* we should now have a string to search in buf[] */
			n = strlen(buf);
			for(cp1 = &buf[0]; n >= slen && cp1 <= &buf[n-slen]; cp1++) {
				if( 0 == strncasecmp(cp1,str,slen) ) {
					/* found a match!! */
					/* printf("Match of %s found in %s\n",str,fname); */
					found = 1;
					break;  /* don't need to search this string anymore */
				}
			}
		}
		if(found) {
			if(! descrip[0]) {
				strcpy(descrip,"Unnamed task");
			}
			if( '1' == hasdate ) {
				sprintf(buf,", %s %02d/%02d/%s",
						completed == '1' ? "Completed":"Due",
						atoi(datemon),atoi(dateday),dateyr);
			}
			else {
				buf[0] = '\0';
			}
#ifdef DEBUG
			printf("%-20.20s %s\n",descrip, buf);
#endif
			{
				char zbuf[1024];
				//sprintf(zbuf,"%-30.30s %s\n",descrip, buf);
				sprintf(zbuf,"%-.80s, Priority %c%s\n",descrip,
						priority, buf);
				zptr->insert(zbuf,'t',fname);
				if( zptr->resultcount >= ZEARCH_MAX_ITEMS ) {
					break;
				}
			}

		}
	}
	fclose(fp);
	if(found) return 1;
	else return 0;
}


void 
scanfiles( Zearch *parent, const char *str )
{
	int i;
	char xmlname[NAME_MAX+1];

	zptr = parent;

	/* get home directory */
	homedir = getenv("HOME");
	if( ! homedir ) {
		homedir = defhome;
	}

	/* search PIM */
	for(i=0;pimlist[i];i++) {
		if( '~' == pimlist[i][0] ) {
			char *cp = pimlist[i];
			cp++; /* point past '~' */
			strcpy(xmlname,homedir);
			strcat(xmlname,cp);
		}
		else {
			strcpy(xmlname,pimlist[i]);
		}
		// printf("Searching %s\n",xmlname);
		if( 0 == i ) {
			zptr->insert("--- Address Book ---",'h',NULL);
			scanxmladdr(xmlname,str);
		}
		else if( 1 == i ) {
			zptr->insert("--- Calendar ---",'h',NULL);
			scanxmldatebk(xmlname,str);
		}
		else if( 2 == i ) {
			zptr->insert("--- To Do Lists ---",'h',NULL);
			scanxmltodo(xmlname,str);
		}
	}

	/* search documents */
	for(i=0;dirlist[i];i++) {
		{
			char buf[NAME_MAX + 20];
			char dname[40];
			dname[0] = '\0';
			if( strstr(dirlist[i],"/cf") ) {
				strcpy(dname,"Compact Flash Card");
			}
			else if( strstr(dirlist[i],"/card") ) {
				strcpy(dname,"SD Card");
			}
			else strcpy(dname,"Documents");

			sprintf(buf,"--- %s ---",dname);
			zptr->insert(buf,'h',NULL);
		}
		if( '~' == dirlist[i][0] ) {
			char dirname[NAME_MAX+1];
			char *cp = dirlist[i];
			cp++; /* point past '~' */
			strcpy(dirname,homedir);
			strcat(dirname,cp);
			searchdir( dirname, str );
		}
		else {
			if( zptr->resultcount < ZEARCH_MAX_ITEMS ) {
				searchdir( dirlist[i], str );
			}
		}
	}
}

