/*
 * $Id: zearch.cpp,v 1.2 2002/08/29 15:56:42 bruce Exp $
 *
 * $Log: zearch.cpp,v $
 * Revision 1.2  2002/08/29 15:56:42  bruce
 *
 * Various changes/bug fixes for release
 *
 */

#include "zearch.h"
#include "zearchlist.h"
#include <qpushbutton.h>
#include <stdio.h>
#include <stdlib.h>
#include <qpe/applnk.h>
#include <malloc.h>
#include <unistd.h>
#include <qpe/qcopenvelope_qws.h>
#include <qmessagebox.h>

extern void scanfiles(Zearch *, const char *);

/* 
 *  Constructs a Zearch which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
Zearch::Zearch( QWidget* parent,  const char* name, WFlags fl )
    : ZearchBase( parent, name, fl )
{
#ifdef ZEARCH_BUTTONS
    connect(sbutton, SIGNAL(clicked()), this, SLOT(search()));
    connect(quit, SIGNAL(clicked()), this, SLOT(goodBye()));
	connect(TextSearch, SIGNAL(returnPressed()), sbutton, SIGNAL(clicked()));
#else
	connect(TextSearch, SIGNAL(returnPressed()), this, SLOT(search()));
#endif
    connect(sbutton, SIGNAL(clicked()), this, SLOT(search()));
	//connect(zl->ZearchListBox, SIGNAL(selectionChanged()), this, SLOT(selected()));
	connect(zl->ZearchListBox, SIGNAL(selected(int)), this, SLOT(selected(int)));
	// allow single click on result to open it
	connect(zl->ZearchListBox, SIGNAL(clicked(QListBoxItem *)), this, SLOT(clicked(QListBoxItem *)));
	itemindex = 0;
	resultcount = 0;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
Zearch::~Zearch()
{
    // no need to delete child widgets, Qt does it all for us
	/*
	int i;
	for(i=0; i<itemindex;i++) {
		free(itemname[i]);
	}
	*/
}

/*
 *  A simple slot... not very interesting.
 */
void Zearch::goodBye()
{
    close();
}


// Search the following:
// Ordinary files (not .desktop, generally .txt, .rtf, .html ...
// 				  ( Possibly spreadsheets and presentations? )
//                 not media files )
// 1) $HOME/Documents/... 
// 2) /usr/mnt.rom/cf/...
// 3) /usr/mnt.rom/card/...
// PIM data:
// 4) $HOME/Applications/addressbook
// 5) $HOME/Applications/datebook/datebook.xml
// 6) $HOME/Applications/todolist/todolist.xml
// Other:
// 7) Mail
// Note PalmOS searches in this order:
// 1) memos, 2)Adddresses, 3) DateBook, 4) Mail, 5) Notes, 6) To Do
// (memos are the text only documents, notes may contain text and/or graphics)
//
// Document Search strategy:
// opendir
// get file
// if ( .desktop file ) {
// 	DocLnk(filename.desktop);
// 	filename = DocLnk.file();
// 	open filename, search for string
// 	if match, put in DocLnk matches list and display
// 	}
// if (directory) search subdirectory
//
// To open selected file:
// DocLnk.execute()  Starts up app for selected doc.
// 
// search function overview:
// create and show results window
// do search, displaying results
// wait for selection of a specific result on the display
// run the associated app for that selection
//
void Zearch::search()
{
	// get search text
	const char *str;

	str = TextSearch->text().latin1();
	//printf("you searched for: %s\n",str);
	//zl->clear();
	zl->ZearchListBox->clear();

	/* free itemname index */
	/* for(i=0; i<itemindex;i++) {
		delete (itemname[i]);
	}
	*/
	itemindex = 0;
	resultcount = 0;
	scanfiles(this,str);
	//
	// TextSearch->setText(""); // Clear search string if desired
	//
	// Display number of results
	{
		char rbuf[255];
		sprintf(rbuf,"<b>Results:</b> %d match%s",resultcount,
				resultcount == 1 ? "":"es");
		zl->ZearchResultLabel->setText( tr(rbuf) );
	}
	zl->ZearchListBox->setFocus();
	TextSearch->setSelection(0,strlen(str));
}

void Zearch::insert(char *str, char itype, char *iname)
{
	if(resultcount >= ZEARCH_MAX_ITEMS)
	{
		fprintf(stderr,"Max search results exceeded!\n");
		return;
	}
	//ZearchBase::zl->insertItem(QString(str), -1);
	zl->insertItem(QString(str), -1);
	itemtype[itemindex] = itype;
	itemname[itemindex] = QString(iname);
	++itemindex;
	if('h' != itype ) { /* not a heading, so increment result count */
		++resultcount;
	}
}

/*
 * perhaps this should be 'pressed' instead of clicked...
 */
void Zearch::clicked(QListBoxItem *item) {
	int i;
	if( item ) {
		i = zl->ZearchListBox->currentItem();
		selected(i);
	}
}

void Zearch::selected(int item)
{
	int i;
	QString text;
	char itype; // d=Document t=todo c=calendar a=addressbook
	char txtbuf[4096];
	char fname[NAME_MAX + 1];
	DocLnk *mydoc;

	//i = zl->ZearchListBox->currentItem();
	i = item;
	text = zl->ZearchListBox->currentText();
	//printf("Slection made, item %d: %s\n",i,text.latin1());
	strncpy(txtbuf,text.latin1(),4095);
	itype = itemtype[i];
	strcpy(fname,itemname[i].latin1());
	//printf("\t type=%c name=%s\n",itype,fname);
	if( 'd' == itype ) { // a document we can open
		mydoc = new DocLnk(fname);
		if( ! mydoc->isValid() ) {
			fprintf(stderr,"Doc file not found!\n");
		}
		else {
			if(mydoc->exec()) {
				mydoc->execute();
				// sleep(2);
				close();
			}
			else {
			 QMessageBox::information( this, "Zearch",
			   "There is no program registered\nto handle this file type.\n");
			}
		}
	}
	else {
		QString c;
		if( 'a' == itype ) { // address
			// execvp("addressbook", NULL);
			c = "addressbook";
		}
		else if( 't' == itype ) { // todo
			//execvp("todolist",NULL);
			c = "todolist";
		}
		else if( 'c' == itype ) { // calendar
			//execvp("datebook",NULL);
			//QString c("datebook");
			c = "datebook";
		}
		else {
			fprintf(stderr,"Doc type %c ignored\n",itype);
			return;
		}
		QCopEnvelope e( "QPE/System", "execute(QString)" );
		e << c;
		close();
	}
}

